@@ -34,7 +34,7 @@ def get_new_dist():
     #meta.DEBUG = True
     meta.current_dist_name = "trusty"
     meta.METARELEASE_URI = \
-        "https://changelogs.ubuntu.com/meta-release-development"
+        "http://changelogs.ubuntu.com/meta-release-development"
     meta.downloaded.wait()
     meta._buildMetaReleaseFile()
     meta.download()
@@ -92,7 +92,7 @@ class TestDistUpgradeFetcherCoreTestCase(unittest.TestCase):
     def testfetcher(self):
         progress = TestAcquireProgress()
         fetcher = TestDistUpgradeFetcherCore(self.new_dist, progress)
-        #fetcher.DEBUG=True
+        #fetcher.DEBUG = True
         res = fetcher.run()
         self.assertTrue(res)
         self.assertTrue(progress.started)
