@@ -18,6 +18,10 @@ from DistUpgrade.DistUpgradeViewNonInteractive import (
 )
 from DistUpgrade import DistUpgradeConfigParser
 from DistUpgrade.utils import url_downloadable
+from DistUpgrade.distro import (
+    UbuntuDistribution,
+    NoDistroTemplateException
+)
 import logging
 import mock
 
@@ -87,7 +91,8 @@ deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 """)
 
-    def test_sources_list_rewrite(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
+    def test_sources_list_rewrite(self, mock_get_distro):
         """
         test regular sources.list rewrite
         """
@@ -96,8 +101,13 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
         apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
+        d.config.set("Distro", "BaseMetaPkgs", "ubuntu-minimal")
+        mock_get_distro.return_value = UbuntuDistribution("Ubuntu", "feisty",
+                                                          "Ubuntu Feisty Fawn",
+                                                          "7.04")
         d.openCache(lock=False)
         res = d.updateSourcesList()
+        self.assertTrue(mock_get_distro.called)
         self.assertTrue(res)
 
         # now test the result
@@ -117,6 +127,94 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
              ]))
 
+    @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
+    def test_sources_list_inactive_mirror(self, mock_get_distro):
+        """
+        test sources.list rewrite of an obsolete mirror
+        """
+        shutil.copy(os.path.join(self.testdir, "sources.list.obsolete_mirror"),
+                    os.path.join(self.testdir, "sources.list"))
+        apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
+        v = DistUpgradeViewNonInteractive()
+        d = DistUpgradeController(v, datadir=self.testdir)
+        d.config.set("Distro", "BaseMetaPkgs", "ubuntu-minimal")
+        mock_get_distro.return_value = UbuntuDistribution("Ubuntu", "feisty",
+                                                          "Ubuntu Feisty Fawn",
+                                                          "7.04")
+        d.openCache(lock=False)
+        res = d.updateSourcesList()
+        self.assertTrue(mock_get_distro.called)
+        self.assertTrue(res)
+
+        # now test the result
+        #print(open(os.path.join(self.testdir,"sources.list")).read())
+        self._verifySources2Way("""
+# main repo
+# deb http://mirror.mcs.anl.gov/ubuntu gutsy main restricted universe multiverse # disabled on upgrade to gutsy
+# deb http://mirror.mcs.anl.gov/ubuntu gutsy-updates main restricted universe multiverse # disabled on upgrade to gutsy
+# deb http://mirror.mcs.anl.gov/ubuntu feisty-proposed main restricted universe multiverse
+# deb http://mirror.mcs.anl.gov/ubuntu gutsy-security main restricted universe multiverse # disabled on upgrade to gutsy
+# deb-src http://mirror.mcs.anl.gov/ubuntu gutsy main restricted universe multiverse # disabled on upgrade to gutsy
+# deb-src http://mirror.mcs.anl.gov/ubuntu gutsy-updates main restricted universe multiverse # disabled on upgrade to gutsy
+##deb-src http://mirror.mcs.anl.gov/ubuntu feisty-proposed main restricted universe multiverse
+# deb-src http://mirror.mcs.anl.gov/ubuntu gutsy-security main restricted universe multiverse # disabled on upgrade to gutsy
+# deb https://example.com/3rd-party/deb/ stable main # disabled on upgrade to gutsy
+deb http://archive.ubuntu.com/ubuntu/ gutsy main
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted universe multiverse # auto generated by ubuntu-release-upgrader
+deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted universe multiverse # auto generated by ubuntu-release-upgrader
+deb http://archive.ubuntu.com/ubuntu gutsy-security main restricted universe multiverse # auto generated by ubuntu-release-upgrader
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted universe multiverse # auto generated by ubuntu-release-upgrader
+deb-src http://archive.ubuntu.com/ubuntu gutsy-updates main restricted universe multiverse # auto generated by ubuntu-release-upgrader
+deb-src http://archive.ubuntu.com/ubuntu gutsy-security main restricted universe multiverse # auto generated by ubuntu-release-upgrader
+""")
+        # check that the backup file was created correctly
+        self.assertEqual(0, subprocess.call(
+            ["cmp",
+             apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".obsolete_mirror",
+             apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
+             ]))
+
+    @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
+    def test_sources_list_no_template(self, mock_get_distro):
+        """
+        test sources.list rewrite when there is no distro template
+        """
+        shutil.copy(os.path.join(self.testdir, "sources.list.obsolete_mirror"),
+                    os.path.join(self.testdir, "sources.list"))
+        apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
+        v = DistUpgradeViewNonInteractive()
+        d = DistUpgradeController(v, datadir=self.testdir)
+        d.config.set("Distro", "BaseMetaPkgs", "ubuntu-minimal")
+        mock_get_distro.side_effect = NoDistroTemplateException("No distro template.")
+        d.openCache(lock=False)
+        res = d.updateSourcesList()
+        self.assertTrue(mock_get_distro.called)
+        self.assertTrue(res)
+
+        # now test the result
+        #print(open(os.path.join(self.testdir,"sources.list")).read())
+        self._verifySources("""
+# main repo
+# deb http://mirror.mcs.anl.gov/ubuntu gutsy main restricted universe multiverse # disabled on upgrade to gutsy
+# deb http://mirror.mcs.anl.gov/ubuntu feisty-updates main restricted universe multiverse # disabled on upgrade to gutsy
+# deb http://mirror.mcs.anl.gov/ubuntu feisty-proposed main restricted universe multiverse
+# deb http://mirror.mcs.anl.gov/ubuntu feisty-security main restricted universe multiverse # disabled on upgrade to gutsy
+# deb-src http://mirror.mcs.anl.gov/ubuntu gutsy main restricted universe multiverse # disabled on upgrade to gutsy
+# deb-src http://mirror.mcs.anl.gov/ubuntu feisty-updates main restricted universe multiverse # disabled on upgrade to gutsy
+##deb-src http://mirror.mcs.anl.gov/ubuntu feisty-proposed main restricted universe multiverse
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted # auto generated by ubuntu-release-upgrader
+deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted # auto generated by ubuntu-release-upgrader
+deb http://security.ubuntu.com/ubuntu gutsy-security main restricted # auto generated by ubuntu-release-upgrader
+# deb-src http://mirror.mcs.anl.gov/ubuntu feisty-security main restricted universe multiverse # disabled on upgrade to gutsy
+""")
+
+        # check that the backup file was created correctly
+        self.assertEqual(0, subprocess.call(
+            ["cmp",
+             apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".obsolete_mirror",
+             apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
+             ]))
+
     def test_commercial_transition(self):
         """
         test transition of pre-gutsy archive.canonical.com archives
@@ -137,7 +235,8 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
 deb http://archive.canonical.com/ubuntu gutsy partner
 """)
 
-    def test_extras_removal(self):
+    @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
+    def test_extras_removal(self, mock_get_distro):
         """
         test removal of extras.ubuntu.com archives
         """
@@ -149,6 +248,9 @@ deb http://archive.canonical.com/ubuntu gutsy partner
                            os.path.join(self.testdir, "sources.list.d"))
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
+        mock_get_distro.return_value = UbuntuDistribution("Ubuntu", "feisty",
+                                                          "Ubuntu Feisty Fawn",
+                                                          "7.04")
         d.openCache(lock=False)
         res = d.updateSourcesList()
         self.assertTrue(res)
@@ -252,7 +354,7 @@ deb http://old-releases.ubuntu.com/ubuntu hoary-security main restricted univers
         "http://us.archive.ubuntu.com/ubuntu", logging.debug),
         "Could not reach mirror")
     def testEOL2SupportedWithMirrorUpgrade(self):
-        " test upgrade from a EOL release to a supported release with mirror"
+        " test upgrade from a EOL release to a supported release with mirror "
         # Use us.archive.ubuntu.com, because it is available in Canonical's
         # data center, unlike most mirrors.  This lets this test pass when
         # when run in their Jenkins test environment.
@@ -387,11 +489,19 @@ deb http://archive.canonical.com/ubuntu gutsy partner
 
     def test_unicode_comments(self):
         """
-        test transition of apt-cacher/apt-torrent uris
+        test a sources.list file with unicode comments
         """
         shutil.copy(os.path.join(self.testdir, "sources.list.unicode"),
                     os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
+        self._verifySources("""
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted
+deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
+deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
+# A PPA with a unicode comment
+deb http://ppa.launchpad.net/random-ppa quantal main # ppa of Víctor R. Ruiz (vrruiz)
+""")
+
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
         d.openCache(lock=False)
@@ -462,7 +572,17 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
                 "expected entry '%s' in sources.list missing. got:\n'''%s'''" %
                 (l, sources_list))
 
-
+    def _verifySources2Way(self, expected):
+        self._verifySources(expected)
+        sources_file = apt_pkg.config.find_file("Dir::Etc::sourcelist")
+        with open(sources_file) as f:
+            sources_list = f.read()
+        for l in sources_list.split("\n"):
+            self.assertTrue(
+                l in expected.split("\n"),
+                "unexpected entry '%s' in sources.list. got:\n'''%s'''" %
+                (l, sources_list))
+                
 if __name__ == "__main__":
     import sys
     for e in sys.argv:
